@@ -382,8 +382,8 @@ gpt_alloc (PedDevice * dev)
 	ped_geometry_init (&gpt_disk_data->data_area, dev, data_start,
 			   data_end - data_start + 1);
 	gpt_disk_data->entry_count = GPT_DEFAULT_PARTITION_ENTRIES;
-	uuid_generate ((char*) &gpt_disk_data->uuid);
-	swap_uuid_and_efi_guid((char*)(&gpt_disk_data->uuid));
+	uuid_generate ((unsigned char*) &gpt_disk_data->uuid);
+	swap_uuid_and_efi_guid((unsigned char*)(&gpt_disk_data->uuid));
 	return disk;
 
 error_free_disk:
@@ -872,8 +872,8 @@ gpt_partition_new(const PedDisk* disk,
 	gpt_part_data->raid = 0;
 	gpt_part_data->boot = 0;
 	gpt_part_data->hp_service = 0;
-	uuid_generate ((char*) &gpt_part_data->uuid);
-	swap_uuid_and_efi_guid((char*)(&gpt_part_data->uuid));
+	uuid_generate ((unsigned char*) &gpt_part_data->uuid);
+	swap_uuid_and_efi_guid((unsigned char*)(&gpt_part_data->uuid));
 	strcpy (gpt_part_data->name, "");
 	return part;
 
