@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2002 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2002, 2004 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,7 +46,8 @@
  *	Apple_Driver		device driver
  *	Apple_Driver43		SCSI Manager 4.3 device driver
  *	Apple_MFS		original Macintosh File System 
- *	Apple_HFS		Hierarchical File System
+ *	Apple_HFS		Hierarchical File System (and +)
+ *	Apple_HFSX		HFS+ with case sensitivity and more
  *	Apple_UNIX_SVR2		UNIX file system (UFS?)
  *	Apple_PRODOS		ProDOS file system 
  *	Apple_Free		unused space
@@ -127,6 +128,7 @@ typedef struct {
 	int		is_driver;
 	int		is_root;
 	int		is_swap;
+	int		is_lvm;
 
 	PedSector	data_region_length;
 	PedSector	boot_region_length;
@@ -413,6 +415,14 @@ _rawpart_is_swap (MacRawPartition* raw_part)
 	return 1;
 }
 
+static int
+_rawpart_is_lvm (MacRawPartition* raw_part)
+{
+	if (strcmp (raw_part->name, "lvm") != 0)
+		return 0;
+	return 1;
+}
+
 static int
 _rawpart_is_void (MacRawPartition* raw_part)
 {
@@ -491,6 +501,7 @@ _rawpart_analyse (MacRawPartition* raw_part, PedDisk* disk, int num)
 	mac_part_data->is_driver = _rawpart_is_driver (raw_part);
 	mac_part_data->is_root = _rawpart_is_root (raw_part);
 	mac_part_data->is_swap = _rawpart_is_swap (raw_part);
+	mac_part_data->is_lvm = _rawpart_is_lvm (raw_part);
 
 	/* "data" region */
 #ifndef DISCOVER_ONLY
@@ -1013,6 +1024,7 @@ mac_partition_new (
 		mac_data->is_boot = 0;
 		mac_data->is_root = 0;
 		mac_data->is_swap = 0;
+		mac_data->is_lvm = 0;
 
 		strcpy (mac_data->volume_name, "untitled");
 	} else {
@@ -1076,7 +1088,8 @@ mac_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		return 1;
 	}
 
-	if (fs_type && !strcmp (fs_type->name, "hfs")) {
+	if (fs_type && (!strcmp (fs_type->name, "hfs")
+			|| !strcmp (fs_type->name, "hfs+"))) {
 		strcpy (mac_data->system_name, "Apple_HFS");
 		mac_data->status |= 0x7f;
 	} else {
@@ -1112,7 +1125,7 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 	case PED_PARTITION_ROOT:
 		if (state) {
 			strcpy (mac_data->volume_name, "root");
-			mac_data->is_swap = 0;
+			mac_data->is_swap = mac_data->is_lvm = 0;
 		} else {
 			if (mac_data->is_root)
 				strcpy (mac_data->volume_name, "untitled");
@@ -1123,7 +1136,7 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 	case PED_PARTITION_SWAP:
 		if (state) {
 			strcpy (mac_data->volume_name, "swap");
-			mac_data->is_root = 0;
+			mac_data->is_root = mac_data->is_lvm = 0;
 		} else {
 			if (mac_data->is_swap)
 				strcpy (mac_data->volume_name, "untitled");
@@ -1131,6 +1144,17 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 		mac_data->is_swap = state;
 		return 1;
 
+	case PED_PARTITION_LVM:
+		if (state) {
+			strcpy (mac_data->volume_name, "lvm");
+			mac_data->is_root = mac_data->is_swap = 0;
+		} else {
+			if (mac_data->is_lvm)
+				strcpy (mac_data->volume_name, "lvm");
+		}
+		mac_data->is_lvm = state;
+		return 1;
+
 	default:
 		return 0;
 	}
@@ -1155,6 +1179,9 @@ mac_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 	case PED_PARTITION_SWAP:
 		return mac_data->is_swap;
 
+	case PED_PARTITION_LVM:
+		return mac_data->is_lvm;
+
 	default:
 		return 0;
 	}
@@ -1168,6 +1195,7 @@ mac_partition_is_flag_available (
 	case PED_PARTITION_BOOT:
 	case PED_PARTITION_ROOT:
 	case PED_PARTITION_SWAP:
+	case PED_PARTITION_LVM:
 		return 1;
 
 	default:
