@@ -1,7 +1,7 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ typedef struct {
 } __attribute__ ((packed)) SunPartitionInfo;
 
 typedef struct {
-	u_int8_t	info[128];	/* Informative text string */
+	char 		info[128];	/* Informative text string */
 	u_int8_t	spare0[14];
 	SunPartitionInfo infos[SUN_DISK_MAXPARTITIONS];
 	u_int8_t	spare1[246];	/* Boot information etc. */
@@ -167,8 +167,9 @@ sun_alloc (PedDevice* dev)
 {
 	PedDisk*	disk;
 	SunRawLabel*	label;
-	PedCHSGeometry*	hw_geom = &dev->hw_geom;
-	PedSector	cyl_size = hw_geom->sectors * hw_geom->heads;
+	SunDiskData*	sun_specific;
+	PedCHSGeometry*	bios_geom = &dev->bios_geom;
+	PedSector	cyl_size = bios_geom->sectors * bios_geom->heads;
 
         disk = _ped_disk_alloc (dev, &sun_disk_type);
 	if (!disk)
@@ -177,33 +178,37 @@ sun_alloc (PedDevice* dev)
 	disk->disk_specific = (SunDiskData*) ped_malloc (sizeof (SunDiskData));
 	if (!disk->disk_specific)
 		goto error_free_disk;
+	sun_specific = (SunDiskData*) disk->disk_specific;
 
-	label = &((SunDiskData*) disk->disk_specific)->raw_label;
+	bios_geom->cylinders = dev->length / cyl_size;
+	sun_specific->length = bios_geom->cylinders * cyl_size;
 
+	label = &sun_specific->raw_label;
 	memset(label, 0, sizeof(SunRawLabel));
 
-	/* util-linux's fdisk asks about these...I'm just going to use
-	   defaults since most people do anyway...sue me.  */
+	/* #gentoo-sparc people agree that nacyl = 0 is the best option */
 	label->magic	= PED_CPU_TO_BE16 (SUN_DISK_MAGIC);
-	label->nacyl	= PED_CPU_TO_BE16 (2);
-	label->pcylcount	= PED_CPU_TO_BE16 (hw_geom->cylinders + 2);
+	label->nacyl	= 0;
+	label->pcylcount	= PED_CPU_TO_BE16 (bios_geom->cylinders);
 	label->rspeed	= PED_CPU_TO_BE16 (5400);
 	label->ilfact	= PED_CPU_TO_BE16 (1);
 	label->sparecyl	= 0;
-	label->ntrks	= PED_CPU_TO_BE16 (hw_geom->heads);
-	label->nsect	= PED_CPU_TO_BE16 (hw_geom->sectors);
-	label->ncyl	= PED_CPU_TO_BE16 (hw_geom->cylinders);
+	label->ntrks	= PED_CPU_TO_BE16 (bios_geom->heads);
+	label->nsect	= PED_CPU_TO_BE16 (bios_geom->sectors);
+	label->ncyl	= PED_CPU_TO_BE16 (bios_geom->cylinders - 0);
 
 	/* Add a whole disk partition at a minimum */
 	label->infos[WHOLE_DISK_PART].id = WHOLE_DISK_ID;
 	label->partitions[WHOLE_DISK_PART].start_cylinder = 0;
 	label->partitions[WHOLE_DISK_PART].num_sectors =
-		PED_CPU_TO_BE32(hw_geom->cylinders * cyl_size);
+		PED_CPU_TO_BE32(bios_geom->cylinders * cyl_size);
 
 	/* Now a neato string to describe this label */
 	snprintf(label->info, sizeof(label->info) - 1,
-		 "GNU Parted Custom cyl %d alt 2 hd %d sec %d",
-		 PED_BE16_TO_CPU(label->ncyl), PED_BE16_TO_CPU(label->ntrks),
+		 "GNU Parted Custom cyl %d alt %d hd %d sec %d",
+		 PED_BE16_TO_CPU(label->ncyl),
+		 PED_BE16_TO_CPU(label->nacyl),
+		 PED_BE16_TO_CPU(label->ntrks),
 		 PED_BE16_TO_CPU(label->nsect));
 
 	sun_compute_checksum(label);
@@ -256,9 +261,9 @@ _check_geometry_sanity (PedDisk* disk, SunRawLabel* label)
 				_("The disk CHS geometry (%d,%d,%d) does not "
 				  "match the geometry stored on the disk "
 				  "label (%d,%d,%d)."),
-				dev->hw_geom.cylinders,
-			       	dev->hw_geom.heads,
-			       	dev->hw_geom.sectors,
+				dev->bios_geom.cylinders,
+			       	dev->bios_geom.heads,
+			       	dev->bios_geom.sectors,
 				PED_BE16_TO_CPU(label->pcylcount),
 				PED_BE16_TO_CPU(label->ntrks),
 				PED_BE16_TO_CPU(label->nsect))
@@ -310,7 +315,7 @@ sun_read (PedDisk* disk)
 		goto error;
 
 	block = disk->dev->bios_geom.sectors * disk->dev->bios_geom.heads;
-	disk_data->length = block * disk->dev->bios_geom.cylinders;
+	disk_data->length = block * PED_BE16_TO_CPU(label->ncyl);
 
 	for (i = 0; i < SUN_DISK_MAXPARTITIONS; i++) {
 		if (!PED_BE32_TO_CPU(label->partitions[i].num_sectors))
@@ -352,7 +357,7 @@ sun_read (PedDisk* disk)
 
 #ifndef DISCOVER_ONLY
 static void
-_probe_and_add_use_old_info (PedDisk* disk)
+_probe_and_use_old_info (PedDisk* disk)
 {
 	SunDiskData*		sun_specific;
 	SunRawLabel		old_label;
@@ -373,13 +378,12 @@ sun_write (PedDisk* disk)
 	SunPartitionData*	sun_data;
 	SunDiskData*		disk_data;
 	PedPartition*		part;
-	PedSector		length;
 	int			i;
 
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (disk->dev != NULL, return 0);
 
-	_probe_and_add_use_old_info (disk);
+	_probe_and_use_old_info (disk);
 
 	disk_data = (SunDiskData*) disk->disk_specific;
 	label = &disk_data->raw_label;
@@ -425,9 +429,18 @@ sun_write (PedDisk* disk)
 	   smaller than it really is, but we'll have that problem even if we
 	   don't do this.  */
 
-	label->pcylcount = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders
-					+ PED_BE32_TO_CPU(label->nacyl));
-	label->ncyl = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders);
+	if (disk->dev->bios_geom.cylinders > 65536) {
+		ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_IGNORE,
+			_("The disk has %d cylinders, which is greater than "
+			  "the maximum of 65536."),
+			disk->dev->bios_geom.cylinders);
+	}
+
+	label->pcylcount = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders);
+	label->ncyl = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders
+			- PED_BE16_TO_CPU (label->nacyl));
 
 	sun_compute_checksum (label);
 
