@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,6 +63,8 @@ _device_get_length (PedDevice* dev)
 static int
 _device_probe_geometry (PedDevice* dev)
 {
+	PedSector cyl_size;
+
 	dev->length = _device_get_length (dev);
 	if (!dev->length)
 		return 0;
@@ -72,12 +74,13 @@ _device_probe_geometry (PedDevice* dev)
 		return 0;
 
 	/* XXX: We have no way to get this!  */
-	dev->sectors = 32;
-	dev->heads = 4;
-	dev->geom_known = 0;
-	dev->cylinders = dev->length / (dev->heads * dev->sectors
-					* (dev->sector_size / PED_SECTOR_SIZE));
-	
+	dev->bios_geom.sectors = 63;
+	dev->bios_geom.heads = 255;
+	cyl_size = dev->bios_geom.sectors * dev->bios_geom.heads;
+	dev->bios_geom.cylinders = dev->length / cyl_size
+					* (dev->sector_size / PED_SECTOR_SIZE);
+	dev->hw_geom = dev->bios_geom;
+
 	return 1;
 }
 
@@ -175,8 +178,6 @@ _init_device (const char *path)
 	dev->external_mode = 0;
 	dev->dirty = 0;
 	dev->boot_dirty = 0;
-	dev->geom_known = 1;
-	dev->geom_already_guessed = 0;
 
 	return dev;
 
@@ -223,7 +224,8 @@ gnu_new (const char* path)
 {
 	PedDevice*	dev;
 	GNUSpecific*	arch_specific;
-	error_t err;
+	error_t         ro_err, rw_err;
+	int             ispath;
 
 	PED_ASSERT (path != NULL, return NULL);
 
@@ -235,18 +237,32 @@ gnu_new (const char* path)
 	arch_specific->consume = 1;
 
  retry_open:
-	err = store_typed_open (dev->path, 0, NULL, &arch_specific->store);
-	if (err) {
-		error_t rw_err = err;
+	/* Try read-write. */
+	if (strchr (dev->path, '/') != NULL) {
+	        /* We set this to prevent having to use strchr more then once. */
+	        ispath = 1; 
 
-		err = store_typed_open (dev->path, STORE_READONLY, NULL,
-					&arch_specific->store);
-		if (err) {
-			if (ped_exception_throw (
-				PED_EXCEPTION_ERROR,
+	        rw_err = store_open (dev->path, 0, NULL, &arch_specific->store);
+	} else { 
+	        rw_err = store_typed_open (dev->path, 0, NULL, &arch_specific->store);
+	}        
+
+	/* Try readonly. */
+	if (rw_err) {
+	  if (ispath) {
+	        ro_err = store_open (dev->path, STORE_READONLY, NULL,
+				 &arch_specific->store);
+	  } else {
+	        ro_err = store_typed_open (dev->path, STORE_READONLY, NULL,
+				       &arch_specific->store);
+	  }
+                 
+	if (ro_err) {
+	        if (ped_exception_throw (
+			        PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_RETRY_CANCEL,
 				_("Error opening %s: %s"),
-				dev->path, strerror (err))
+				dev->path, strerror (ro_err))
 					!= PED_EXCEPTION_RETRY) {
 				return NULL;
 			} else
@@ -844,6 +860,7 @@ static PedDeviceArchOps gnu_dev_ops = {
 	write:		gnu_write,
 	check:		gnu_check,
 	sync:		gnu_sync,
+	sync_fast:	gnu_sync,
 	probe_all:	gnu_probe_all
 };
 
